from tkinter import filedialog, StringVar


def select_file(var: StringVar, initial_dir="/"):
    filetypes = (
        ('All files', '*.*'),
    )

    filename = filedialog.askopenfilename(
        title='Select vpn cli file',
        initialdir=initial_dir,
        filetypes=filetypes)
    if filename:
        var.set(filename)



from pathlib import Path
from sys import argv

import click as click

from dvpn.config.constants import PublicVars
from dvpn.config.paths import secret_path
from dvpn.modules.tools import connect
from dvpn.modules.vpncli import VpnCli
from dvpn.modules.windows import open_gui


@click.group()
def cli():
    pass


@cli.command(name="connect")
@click.argument("host", nargs=1)
def _connect(host):
    connect(host)


@cli.command(name="disconnect")
def _disconnect():
    VpnCli.reset()


@cli.command(name="autoresolve")
def _auto():
    print("This is not implemented yet")
    VpnCli.check_accessed()


@cli.command()
def gui():
    open_gui()


def main():
    if "gui" not in argv:
        if not secret_path.exists():
            print(
                f"Please first Create secret.json according to README at"
                f" {str(Path(__file__).resolve().parent / 'config')} or use dvpn gui")
            exit(1)
        else:
            PublicVars()

        if len(PublicVars().credentials.keys()) == 0:
            print(
                "No Credentials Found please make sure to add your VPNs before "
                "continuing "
                "with dvpn gui")
    cli()



from tkinter import Tk

from dvpn.config.constants import PublicVars
from dvpn.modules.vpncli import VpnCli

buttons = []


def state_buttons(state_out):
    for btn in buttons:
        btn.config(state=state_out)


def clear_buttons():
    for btn in buttons[:-1]:
        btn.config(bg="darkgray", fg="white")


def reopen(last_window: Tk):
    last_window.destroy()
    from dvpn.modules.windows import open_gui
    open_gui()


def connect(host) -> bool:
    creds = PublicVars().credentials[host]
    vpncli = VpnCli(str(creds["cli_path"]))
    return vpncli.connect(creds)



import logging
import os
import re
import subprocess
import sys
from typing import Union, Any, Match

from dvpn.config.constants import PublicVars
from dvpn.config.paths import vpn_cli_file_path_win, vpn_cli_file_path_osx

if sys.platform == "win32":
    import wexpect
else:
    import pexpect as wexpect


class VpnCli:
    def __init__(self, cli_path):
        print("\nWelcome to DieVPN\n")
        print(
            "Make sure to kill all VPN clients before usage, as cli would collide with it"
        )
        self.process_pipe = None
        self.cli_path = cli_path
        print("Resetting connection for future stability")
        self.reset()

    def get_connected_vpn(self):
        stdout = subprocess.check_output([self.cli_path, "stats"]).decode()
        # Match and remove dot on end
        vpn_name_index = re.findall(".*onnected to*. .*", stdout)[0][:-1]

        return vpn_name_index.split(" ")[1]

    @classmethod
    def reset(cls, cli_path=None):
        print("...Disconnecting")
        # TODO: fix this for more types of vpn supported
        pipe = wexpect.spawn(
            command=str(vpn_cli_file_path_win)
            if os.name == "nt"
            else str(vpn_cli_file_path_osx),
            args=["disconnect"],
        )
        output = pipe.readlines()

        if "disconnected" in "".join(output[-3:-1]).lower():
            print("Disconnected")
        else:
            print("Not Connected")

    def __connect(self, host, username, password, **kwargs):
        print(f">> Connecting to {host}")
        self.process_pipe = wexpect.spawn(
            command=self.cli_path, args=["connect", f"{host}"]
        )

        print("     ... Waiting for VPN to complete its chores")
        self.process_pipe.expect(".*sername*.")
        print("     > Entering Username")
        self.process_pipe.sendline(username)
        self.process_pipe.expect(".*assword*.")
        print("     > Entering Password")
        self.process_pipe.sendline(password)

        if kwargs.get("banner", False):
            self.process_pipe.expect(".*accept*.")
            self.process_pipe.sendline("y")

        output = self.process_pipe.readlines()
        logging.info("".join(output))
        return "connected" in "".join(output[-3:-1]).lower()

    def connect(self, creds: dict, banner: bool = False) -> bool:
        print("~ Resetting connection")
        self.reset()
        if self.__connect(**creds):
            print("<Successfully Connected>")
            return True
        else:
            print("Connection failed :(")
            return False

    @classmethod
    def check_containing(
        cls, read_lines: str, searched_list: tuple
    ) -> Union[
        Match[str], None, Match[Union[Union[str, bytes], Any]], tuple[bool, None]
    ]:
        for url in searched_list:
            return re.findall(url, read_lines)
        return False, None

    @classmethod
    def check_accessed(cls):
        out_dict = dict()
        if sys.platform == "win32":
            stdout = subprocess.check_output(["ipconfig", "/displaydns"]).decode()
            print(stdout)
            credentials = PublicVars().credentials
            for key in credentials.keys():
                out_dict[key] = VpnCli.check_containing(
                    stdout, credentials[key]["urls"]
                )  # Check if A server record error



from json import JSONDecodeError

from dvpn.config.constants import PublicVars
from dvpn.config.paths import secret_path


def add_credentials(cli_path: str, name: str, host: str, user: str, pwd: str,
                    banner: bool):
    if not name:
        return False

    exists = secret_path.exists()

    credentials = dict()

    if exists:
        try:
            credentials = PublicVars().credentials
        except JSONDecodeError:
            print("Invalid JSON!")

    credentials[name] = {
        "cli_path": cli_path,
        "host": host,
        "username": user,
        "password": pwd,
        "banner": banner,
    }

    PublicVars().credentials = credentials

    return True



from threading import Thread
from tkinter import Tk, Button

from dvpn.config.constants import default_title
from dvpn.modules.tools import state_buttons, clear_buttons, connect
from dvpn.modules.vpncli import VpnCli


def connect_threaded(window, host, instigator):
    Thread(target=_connect_threaded, args=[window, host, instigator]).start()


def disconnect_threaded(window: Tk):
    Thread(target=_disconnect_threaded, args=[window]).start()


def _connect_threaded(window: Tk, host, instigator: Button):
    window.title(f"DieVPN Connecting to {host}")
    state_buttons("disabled")
    clear_buttons()
    if connect(host):
        instigator.config(bg="darkgreen")
        window.title(f"DieVPN Connected to {host}")
    else:
        instigator.config(bg="darkred")
        window.title("Die VPN Control")
    state_buttons("normal")


def _disconnect_threaded(window: Tk):
    window.title("Disconnecting")
    VpnCli.reset()
    window.title(default_title)
    clear_buttons()



import json
from builtins import dict

from dvpn.config.paths import secret_path

default_title = "Die VPN Control"


class PublicVars:
    _credentials = {}
    instance = None

    def __init__(self):
        self._credentials = {}

    def __new__(cls, *args, **kwargs):
        if cls.instance is None:
            return object.__new__(cls)
        else:
            return cls.instance

    @property
    def credentials(self) -> dict:
        if not secret_path.exists():
            with open(str(secret_path), "w") as fp:
                json.dump({}, fp, indent=1)
        with open(str(secret_path), "r") as fp:
            _credentials = json.load(fp)
        return _credentials

    @credentials.setter
    def credentials(self, updated: dict):
        with open(str(secret_path), "w") as fp:
            json.dump(updated, fp, indent=4)



from pathlib import Path

secret_path = Path(__file__).resolve().parent.parent / "config" / "secret.json"

vpn_cli_file_path_win = Path("C:\\") / \
                        "Program Files (x86)\\" / "Cisco\\" / \
                        "Cisco AnyConnect Secure Mobility Client\\vpncli.exe"

vpn_cli_file_path_osx = Path("/opt/cisco/anyconnect/bin/vpn")



import os
from pathlib import Path
from tkinter import (
    Tk,
    StringVar,
    BooleanVar,
    Label,
    Entry,
    Checkbutton,
    Button,
    Toplevel,
    Frame,
    RIGHT,
    LEFT, Menu,
)

from dvpn.config.constants import default_title, PublicVars
from dvpn.config.paths import vpn_cli_file_path_osx, vpn_cli_file_path_win
from dvpn.modules.commands import select_file
from dvpn.modules.handles import add_credentials
from dvpn.modules.threaded_funcs import connect_threaded, disconnect_threaded
from dvpn.modules.tools import buttons, reopen


def new_vpn_window(root):
    window = Toplevel(root)
    window.title("Modify or add VPNs")

    file_name = StringVar()
    file_name.set(
        str(vpn_cli_file_path_win) if os.name == "nt" else str(vpn_cli_file_path_osx)
    )

    file_frame = Frame(window)
    name = StringVar()
    host = StringVar()
    user = StringVar()
    pwd = StringVar()
    banner_var = BooleanVar()

    window.geometry(f"300x300+10+20")

    file_path = Entry(file_frame, textvariable=file_name)
    fp_btn = Button(
        file_frame,
        text="Open",
        command=lambda file_tmp=str(Path(file_name.get()).parent): select_file(
            file_name,
            file_tmp,
        ),
    )
    lbl_clipath = Label(file_frame, text="VPN Cli Path")
    lbl_name = Label(window, text="Name")
    text_name = Entry(window, textvariable=name)
    lbl_host = Label(window, text="Host")
    text_host = Entry(window, textvariable=host)
    lbl_user = Label(window, text="Username")
    text_user = Entry(window, textvariable=user)
    lbl_pwd = Label(window, text="Password")
    text_pwd = Entry(window, textvariable=pwd)
    banner = Checkbutton(window, text="Has Banner", variable=banner_var)

    submit_btn = Button(window, text="Apply", foreground="white", bg="gray")

    lbl_clipath.pack(side=LEFT)
    file_path.pack(expand=True, fill="x", side=LEFT)
    fp_btn.pack(side=RIGHT)
    file_frame.pack(expand=True, fill="x")
    lbl_name.pack()
    text_name.pack(expand=True)
    lbl_host.pack()
    text_host.pack(expand=True)
    lbl_user.pack()
    text_user.pack(expand=True)
    lbl_pwd.pack()
    text_pwd.pack(expand=True)
    banner.pack(expand=True)
    submit_btn.pack(expand=True, fill="x")

    submit_btn.bind(
        "<Button-1>",
        lambda event: add_credentials(
            file_name.get(),
            name.get(),
            host.get(),
            user.get(),
            pwd.get(),
            banner_var.get(),
        ),
    )

    window.mainloop()


def open_gui():
    print("Make sure to kill all VPN clients before usage, as cli would collide with it")
    window = Tk()
    # add widgets here
    main_menu = Menu(window, tearoff=0)
    main_menu.add_command(label="Add VPN", command=lambda: new_vpn_window(window))
    main_menu.add_command(label="Refresh", command=lambda: reopen(window))
    window.config(menu=main_menu)
    window.title(default_title)
    keys = PublicVars().credentials.keys()
    window.geometry(f"300x{(len(keys) + 1) * 35}+10+20")
    for vpn_connection in keys:
        btn = Button(text=vpn_connection.capitalize(), bg="darkgray", fg="white")
        btn.bind('<Button-1>',
                 lambda event, con=vpn_connection, tbtn=btn: connect_threaded(window, con,
                                                                              tbtn))
        buttons.append(btn)
        btn.pack(expand=True, fill="x")
    btn = Button(text="Disconnect", bg="black", fg="white",
                 command=lambda: disconnect_threaded(window))
    btn.pack(expand=True, fill="x", pady=5)
    buttons.append(btn)
    window.mainloop()
